Introduction
-----------

Siesta is a stress-free JavaScript unit and UI-testing tool. It is known that change of activity is a form of rest,
so stop writing code, write some tests and have some rest! Your application will win from both.

Siesta is very easy to learn and as your test suite grows and your requirements becomes more complex it still scales very well. 

Siesta is also cross-platform - the same tests can run in browsers and NodeJS, on Linux, MacOs and Windows 
(assuming of course they are written in a platform-independent manner). Headless browser modes are supported.

{@img images/demo.png autogen}

In this guide, we assume a setup, pointed toward running tests in browsers.

For the setup, targeting NodeJS environment, please refer to this guide <a href="#!/guide/bdd_conventions">Siesta getting started with NodeJS.</a>


Siesta browser project
--------------

Since plain browsers can't access the file system, we need to start with creation of the Siesta project - a single place for storing the meta-information
about your test suite. 


    const project = new Siesta.Project.Browser();
    
    project.configure({
        title       : 'Basic browser test suite',
        
        preload     : [
            'https://code.jquery.com/jquery-1.6.4.min.js'
        ]
    });
    
    project.plan(
        '010_sanity.t.js',
        '020_basic.t.js'
    );
    
    project.start();


Lets save this file as `siesta.js` in the `tests` folder of your app. We'll need a wrapping html file for this project too, to be able to access
the web interface, lets name it `siesta.html`:

	<!DOCTYPE html>
    <html>
        <head>
            <!-- Recommended set of pragmas, required for IE11 compatibility-->
            <meta http-equiv="X-UA-Compatible" content="IE=edge">
            <meta http-equiv="content-type" content="text/html; charset=UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1, user-scalable=no">
            
            <!-- Web interface -->
            <link rel="stylesheet" type="text/css" href="__SIESTA_FOLDER__/resources/css/siesta-all.css">
            <script type="text/javascript" src="__SIESTA_FOLDER__/siesta-all.js"></script>
            
            <!-- Project file -->
            <script type="text/javascript" src="siesta.js"></script>
        </head>
        
        <body>
        </body>
    </html>

As you've probably already guessed, we've created a project with 2 test files: `010_sanity.t.js` and `020_basic.t.js`. **The paths given, are relative
to the `siesta.html` wrapper file**

Lets check how the test file look itself:

    StartTest(t => {
    
        t.it("Sanity", t => {
        
            t.ok($, 'jQuery is here');
            
            t.is(1, "1", "Relaxed equality")
            
            t.exact(1, 1, "Exact equality")
            
            t.is(1, t.anyNumberApprox(0.95, 0.1), "Fuzzy equality")
            
            t.like(window.location.href, /010_sanity\.t\.js/, "Location of the test file is as expected")
            
            t.throwsOk(
                () => { vixie },
                /vixie/,
                "Correct exception thrown"
            )
        });
    })    

The test code is wrapped with `StartTest(t => { ... })` construct. The function passed to it will receive an instance 
of the {@link Siesta.Test} class, as the 1st argument.

The instance of the test class has various *assertion* which are just methods of the class. An "assertion" is any statement about your code. 
It may have arbitrary meaning, ranging from very simple like "this variable is equal to that variable" to complex and domain specific: 
"this instance of Observable will fire this event exactly N times during the following X milliseconds". 

Siesta has many general-purpose assertions built-in and it also allows you to <a href="#!/guide/71_extending_test_class">create your own assertions.</a>
The statement of an assertion can be either truthy (we say "assertion pass") or false (assertion fail).

In the example above we use the simplest possible assertion: `t.ok(value, description)`. It passes when the provided `value` is "truthy", 
like : `true / 1 / "TRUE" / {}` etc and fails otherwise. Each assertion also has an optional description which should contain 
an explanation of what's being asserted.

We also used `throwsOk` assertion, which, accepts a function, run it, and verify that it does throw exception, which serializes to the string, 
matching a given `RegExp`. You've probably get the idea. 

**To see the list of all generic assertions, supported by Siesta**, please refer to: {@link Siesta.Test Siesta.Test} class. 

**For the list of browser-specific assertions**, please refer to: {@link Siesta.Test.Browser Siesta.Test.Browser} class.
 
Ok, we now have a project file, a web-interface wrapper for it, and test file. Lets open the `siesta.html` file in a browser.

<p class="side-note">
While it is possible to use file-system only setup, using `file://` urls, it is not recommended, as support for it varies per browser
and some features may be different from normal `http://` setup. 
<br><br>
The recommended way is to use a local web server on your machine. For a simple, zero-configuration one, just to serve the static files, you can check
this Node package: <a href="https://www.npmjs.com/package/local-web-server">https://www.npmjs.com/package/local-web-server</a>
<br><br>
So, the browser url should look like: http://localhost/yourproject/tests
</p>

You should see something like this:

{@img images/synopsys.png autogen}

Click the "Run all" button in the toolbar. Notice, how the `010_sanity.t.js` test gains a green checkmark, indicating it has completed
without failing assertions.

The `020_basic.t.js` has a red cross indicating the file is missing - feel free to create it after reading a couple of the following sections :)


Sandboxing. Test page vs project web-interface page
-----------

By default, {@link Siesta.Project.Browser#sandbox sandboxing} is enabled. Every test is launched in isolated browser JavaScript context. Isolation is
done using iframes or {@link Siesta.Project.Browser#runInPopup popups}. In general, there is no need to cleanup anything after the test.
 
This behavior is configurable with the {@link Siesta.Project.Browser#sandbox sandbox} option, but we recommend you keep it enabled
until you start feeling comfortable with Siesta. 

One need to understand the difference between the test page - the iframe in which the test will be launched, and the project page - the web-interface
page. These are not related, and run different code.

Typical mistake is to include the files from your application to the web-interface page:

    <!-- Web interface -->
    <link rel="stylesheet" type="text/css" href="__SIESTA_FOLDER__/resources/css/siesta-all.css">
    <script type="text/javascript" src="__SIESTA_FOLDER__/siesta-all.js"></script>
    
    <!-- WRONG, web-interface does not need files from your projects -->
    <link rel="stylesheet" type="text/css" href="my/project/styles.css">
    <script type="text/javascript" src="my/project/bundle.js"></script>
    
    <!-- Project file -->
    <script type="text/javascript" src="siesta.js"></script>

In the same manner don't include the Siesta files in the {@link Siesta.Project.Browser#preload preload} option:

    project.configure({
        title       : 'Awesome Test Suite',
        
        preload     : [
            '../../siesta-all.js', // WRONG, no need to include siesta files on the test page
            '../my-app-all.js'
        ]
    });
    

Configuring the project
---------

A project can be configured with the {@link Siesta.Project.Browser#configure configure} method, which can be called several times. 
See the {@link Siesta.Project.Browser} for a detailed description of all available options.
The most important option is {@link Siesta.Project.Browser#preload preload}, which defines 
the files that should be pre-loaded for each test. All urls in {@link Siesta.Project.Browser#preload preload} config 
are relative to the project html wrapper file `siesta.html`. 

You can add test files to the project using the {@link Siesta.Project.Browser#plan plan} method, which can be called several times. 

Note, that the project file contains regular javascript code, so you can configure it 
differently, depending on various conditions. Siesta includes [Bowser](https://github.com/ded/bowser)
library for browser detection, so you can for example mute some tests for certain browsers: 

    var isDev   = window.location.href.match(/localhost/);
    var project = new Siesta.Project.Browser()
    
    project.configure({
        title       : 'Awesome Test Suite',
        
        preload     : [
            isDev ? '../ext-4.0.6/ext-all-debug.js' : '../ext-4.0.6/ext-all.js',
            isDev ? '../yourproject-all-debug.js' : '../yourproject-all.js'
        ]
    });
    
    project.plan('some_test_for_all_browsers.t.js')
    
    if (!bowser.msie) project.plan('some_test_for_all_browsers_except_IE.t.js');
    
    project.start()


Once the project is fully configured, you can launch it with the {@link Siesta.Project.Browser#start start} method.

As your test suite grows, you may need to start grouping your tests in a logical hierarchy. You can do this by passing special 
"test file descriptor" instead of the string to the {@link Siesta.Project.Browser#plan plan} method.
The descriptor should contain a group name and an array of child descriptors: 

    project.plan(
        '011_single.t.js',
        {
            group       : 'Rendering tests',
            
            items       : [
                'rendering/grid.t.js',
                'rendering/tree.t.js'
            ]
        }
    );
    
In turn, child descriptors can be groups as well. This feature is especially useful, when you need to override the 
{@link Siesta.Project.Browser project} options for some group of tests (see the following section).

See also {@link Siesta.Project.Browser#plan} method for additional information.


Configuring the individual test
---------

When configuring a "test file descriptor", one can also provide some of the project configuration options and they 
will override the corresponding options that were provided to the project.
These options are explicitly marked in the {@link Siesta.Project.Browser project} documentation. 
For example, one can have a test with its own `preload` and `autoCheckGlobals` configs: 

    project.configure({
        autoCheckGlobals    : false
        ...
    })
    
    project.start(
        {
            url     : '011_simple.t.js',
            
            autoCheckGlobals : true,
            preload : [
                ...
            ]
        },
        '012_complex.t.js'
    )

When specifying config options for a group descriptor, these options will override the configuration for all child descriptors of that group:

    project.start(
        {
            group       : 'On demand loading',
            
            // will override the `preload` option for all tests in this group
            preload     : [
                ...
            ],
            
            items       : [
                ...
            ]
        }
    )
    
You can also provide test file descriptor in the test file itself, by adding it to the StartTest call:

    StartTest({
        autoCheckGlobals : false
    }, t => {
        ...
    }) 
    
Values from this object takes the highest priority and will override any other configuration.  

See also {@link Siesta.Project.Browser#plan} method for additional information.


Simulating user interaction
---------

Siesta can simulate user interactions such as {@link Siesta.Test.Browser#click click}, {@link Siesta.Test.Browser#doubleClick double click}, 
{@link Siesta.Test.Browser#type type}, {@link Siesta.Test.Browser#dragBy drag-drop} etc. 

Mouse simulating user actions are listed here: {@link Siesta.Test.UserAgent.Mouse}
Keyboard simulating user actions are listed here: {@link Siesta.Test.UserAgent.Keyboard}
Touch simulating user actions are listed here: {@link Siesta.Test.UserAgent.Touch}

By default Siesta uses synthetic events simulation, which is very fast. 

In some cases, however, you may need to test some intrinsic browser behavior which synthetic events can not reproduce. 
Most notable example will be testing the `:hover` CSS styles of some DOM elements.
In such case you can use {@link Siesta.Project.Browser#simulation native events simulation}

Keep in mind, for tests involving mouse interaction - you should not move the mouse during the execution of such test, 
to not interfere with simulation.


User actions recorder
---------

You can also record user actions as Siesta test. Please refer to this guide: <a href="#!/guide/event_recorder">Using the event recorder.</a>









Limitations 
-----------

Please note that the project page is limited by the same-origin policy as any other web page. You can include scripts/CSS from another domain, but XHR requests on older browsers
are restricted to the same domain. The best approach is to run the test suite on the same domain as your application. You can also configure a server side proxy. 












Writing tests in BDD style
------------

Siesta supports BDD syntax for writing your tests. Please refer to this guide for more information: <a href="#!/guide/bdd_conventions">Writing tests in BDD style.</a>


    



Assertions
---------

Some assertions are generic and cross platform - they belong to the {@link Siesta.Test} class. Others are browser/ExtJS specific - and those are 
listed in the {@link Siesta.Test.Browser} / {@link Siesta.Test.ExtJS} classes.

When an assertion passes - it shows a green checkmark with the assertion description (if description is not provided, Siesta will try to generate 
a sensible default text). When it fails, it also tries to provide you with as much information about the failure as possible, including the 
arguments passed to the assertion method. This is why you are encouraged to use various special assertions for each specific case. For example,
lets say we would like to check if one value is greater than other. We could do that with a simple:

    t.ok(value1 > value2, 'Value1 is greater than value2')
    
But in case of a failure, the only additional information you will see will be something like:

    Failed assertion [ok] at line xxx, 
    Got  : false, 
    Need : "truthy" value"
    
Compare with the output from the more specific "isGreater" assertion: 

    t.isGreater(value1, value2, 'Value1 is greater than value2')

It will output the message along with the provided arguments, instantly making it clear what happened:
    
    Failed assertion [isGreater] at line xxx, 
    Got                : value1, 
    Need, greater than : value2
    
So make sure you've scanned the documentation {@link Siesta.Test} or {@link Siesta.Test.ExtJS} - doing so will save you a lot of time.

At this point you should have enough information to start - feel free to leave this document and start hacking your test suite together :)


User interface
---------

The project UI is stateful - it keeps the selected row, checked rows and options in cookies. You can refresh the project page anytime and your settings will remain.

The rows in the test files grid has a context menu:
{@img images/ui-context-menu.png autogen}

The "Options" button in the toolbar contains several checkboxes corresponding to project {@link Siesta.Project.Browser configuration options}:
{@img images/ui-options-button.png autogen}  


##Testing asynchronous code

When testing asynchronous code, the test function (passed to `StartTest`) may complete its execution, but the test itself
will still be running and incoming assertions will be valid. So, you need to explain to the test, that it needs to wait some time before finalizing.

For that, before some asynchronous part of the code is started, indicate its beginning with {@link Siesta.Test#beginAsync beginAsync} or 
{@link Siesta.Test#wait wait} methods. Then, once the async code complete, use {@link Siesta.Test#endAsync endAsync} 
or {@link Siesta.Test#endWait endWait} accordingly.

Both beginAsync and wait methods will start an "asynchronous code frame", the only difference is that `beginAsync` will return you an auto-generated id for it
(which you need to save to some variable), and `wait` need to receive an unique id as parameter.

For example:

    var async = t.beginAsync();
    
    Ext.Ajax.request({
        url     : 'ajax_demo/sample.json',
        
        success : function (response, opts) {
            t.is(response, 'foobar', 'Response is correct');
            
            t.endAsync(async);
        },
        
        failure : function (response, opts) {
            t.fail("request failed");
            
            t.endAsync(async);
        }
    });
    
Or the same with `wait`:

    t.wait('xhrSample');
    
    Ext.Ajax.request({
        url     : 'ajax_demo/sample.json',
        
        success : function (response, opts) {
            t.is(response, 'foobar', 'Response is correct');
            
            t.endWait('xhrSample');
        },
        
        failure : function (response, opts) {
            t.fail("request failed");
            
            t.endWait('xhrSample');
        }
    });

You can start as many asynchronous code "frames" as you need. By default, all frames will be forced to finalize after 15 seconds, so the whole test
will not get stuck in case of unexpected failures. You can configure this interval in the {@link Siesta.Test#beginAsync beginAsync} and {@link Siesta.Test#wait wait} calls.

Note, that you don't need to use these methods for built-in Siesta methods that accepts callbacks - Siesta already does that for you:

    // the "click" method is asynchronous and accepts a callback
    // there's no need for `beginAsync` though, since its a built-in method
    t.click('.some-button', function () {
    }) 
    
Additionally, if you return a `Promise` instance from the test function itself, Siesta will wait until that promise is resolved before finalizing the test.
In modern browsers, this allows us to use `async/await` functions:

    StartTest(t => {
        let someAsyncOperation = () => new Promise((resolve, reject) => {
            setTimeout(() => resolve(true), 1000)
        })
    
        t.it('Doing async stuff', async t => {
            let res = await someAsyncOperation()
    
            t.ok(res, "Async stuff finished correctly")
        })
    }) 


### Chaining

Many methods of the test class are asynchronous and accept a callback. When you need to execute one such method after another, 
the nesting level can grow quite quickly and affect the readability of the test code:

    t.type(userNameField, 'username', function () {
        t.type(passwordField, 'secret', function () {
            t.click(loginButton, function () {
                // done
            })
        })
    })
    
If you see such code in your tests, make sure you've checked the {@link Siesta.Test#chain chain} method. It allows you keep the nesting to a minimum:

    t.chain(
        function (next) {
            t.type(userNameField, 'username', next)
        },
        function (next) {
            t.type(passwordField, 'secret', next)
        },
        function (next) {
            t.click(loginButton, next)
        },
        function () {
            // done
        }
    })
    
Or even more compact notation, providing action configuration objects instead of functions:

    t.chain({
        action  : 'type',
        target  : userNameField, 
        text    : 'username'
    },{
        action  : 'type',
        target  : passwordField, 
        text    : 'secret'
    },{
        action  : 'click',
        target  : loginButton 
    })

 
### Waiting

Since the nature of web applications is very asynchronous (Ajax calls, animations, etc.), be prepared to wait a lot in your tests. 
To wait for a condition, do:

    this.waitFor(
        // The condition to check for, this line will mean waiting until the presence of #foo element
        function() { return document.getElementById('foo'); },  

        // The callback, after the condition has been fulfilled
        function(el) { /* DO COOL STUFF */ }	      
    );
    
You will find lots and lots of waitForXXX methods in the API to assist you in various situations, example:

	t.waitForSelector('.some_class', function() {
        // Found it
	});

    document.body.className = 'some_class';	// this fulfills the condition, 
                                            // and the callback function is called

You can also wait easily in a chain, by adding a "waitFor" step with a value equal to XXX of any waitForXXX command as the value:

	t.chain({
		{ waitFor : 'selector', args : ['.some_class'] },	// calls waitForSelector which waits until 
		                                                    // the some_class CSS class exists in the DOM
		
		{ waitFor : 500 },	                                // waits for 500 ms

		{ waitFor : 'elementVisible', args : [someEl] },	// calls waitForElementVisible and waits 
		                                                    // for an element to become visible
	 });



Detecting global variable leaks
---------

Siesta has a special built-in assertion, called {@link Siesta.Test#verifyGlobals t.verifyGlobals()}. It will scan the global properties of the test 'window' object,
and compare them with the properties of a clean and fresh scope (properties from an empty `<iframe>`). In case it finds any "unexpected" globals
it will report them as a test failure. You can specify your list of additional "expected" globals, using the {@link Siesta.Project.Browser#expectedGlobals expectedGlobals} configuration
option of the project, or by using the {@link Siesta.Test#expectGlobals t.expectGlobals} method of your test.

You can enable this assertion to be executed automatically at the end of each test, by setting {@link Siesta.Project.Browser#autoCheckGlobals autoCheckGlobals} to true on the project configuration.

For example, in project:

    project.configure({
        autoCheckGlobals    : true,
        expectedGlobals     : [ 'Ext', 'MyProject' ],
        ...
    });

And then in tests:

    // will suppress the complaints about these globals
    t.expectGlobals('Additional', 'Globals');

Please note however, that this feature is not supported in IE <= 8, due to a bug in its JavaScript implementation: <http://social.msdn.microsoft.com/Forums/en/iewebdevelopment/thread/065a5b93-6474-4756-aaea-1fd55d4a013a>
Also see: <http://blogs.msdn.com/b/ericlippert/archive/2005/05/04/414684.aspx>






"TODO" assertions
---------

Sometimes you might want to mark some assertions in a test as "TODO". For example, you might start to write a test covering an edge case,
but only fully implement it later. It's still desirable to run such assertions (sometimes they can accidentally start passing actually), 
but if they fail - that doesn't mean a failure of the whole test.

In such cases you can wrap your assertions using the {@link Siesta.Test#todo t.todo()} method:

    t.todo('Scheduled for 4.1.x release', function (todo) {
    
        var treePanel    = new Ext.tree.Panel()
    
        todo.is(treePanel.getView().store, treePanel.store, "NodeStore and TreeStore have been merged and there's only 1 store now");
    })
    
    // or allow an assertion to fail until a certain date
    t.snooze(new Date(2016, 0, 1), function(t) {
        // TODO fix this soon
        t.expect(1).toBe(2);
    });
    
As you can see, the `todo` method accepts a description and a new test function. That function will receive a new "TODO" test instance, 
as the 1st argument (you can name it arbitrarily). Any failures coming from the methods of this new test instance 
will be reported as "green", as they are expected.

See also {@link Siesta.Test#snooze snooze} method.
 

Extending the test class
---------

Quite often you may find yourself repeating various initialization code in your tests. Or you may have your own assertions, 
specific to your data. To avoid repeation, you can extend the default Siesta test class by adding own methods to it. 
Please refer to <a href="#!/guide/extending_test_class">this guide</a> to know more.

Automation
---------

You can run your tests from the command line and generate various reports with test suite results. 

Please refer to <a href="#!/guide/siesta_automation">Automation Guide.</a> for more information.  


Best practices
---------

In general, you can organize your tests suite as you prefer - there are no strict rules where to place the files, 
how to name them, etc. Choose the way that is more appropriate and efficient in your case. 

There are, however, some simple guidelines. First of all, treat your test suite as you treat your main codebase. 
After all *it is* a part of your codebase, just not visible to the end user. You will need to maintain it, 
update tests after refactoring (refactoring having a test suite is very pleasant experience btw) and so on.

So, use the same coding style and practices as in your main codebase, provide meaningful assertion descriptions, 
write comments in non-trivial places, avoid repeating code and keep your test suite up-to-date.

It is also a good idea to structure your tests into independent sections. Please refer to <a href="#!/guide/bdd_conventions">BDD guide</a> for details


Advanced setup
---------

For own user interface, which is loaded on the project page, Siesta uses an optimized build of ExtJS, compiled with Sencha Cmd.
Only the classes, actualy used in the Siesta UI, are included in the "siesta-all.js" file. If you found that you need
some extra ExtJS class in your project, you can use alternative setup, as shown below. 
Note, that Siesta UI requires ExtJS version 6.0.1 precisely.   

    <!DOCTYPE html>
    <html>
        <head>
            <meta http-equiv="content-type" content="text/html; charset=UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1, user-scalable=no">
            
            <title>Sample project</title>
            
            <link rel="stylesheet" type="text/css" href="__SIESTA_FOLDER__/resources/css/siesta-all.css">
            
            <script type="text/javascript" src="__EXTJS_6.0.1_FOLDER__/build/ext-all-debug.js'"></script>
            <script type="text/javascript" src="__EXTJS_6.0.1_FOLDER__/build/classic/theme-triton/theme-triton.js"></script>
            <script type="text/javascript" src="__SIESTA_FOLDER__/siesta-no-ext-all.js"></script>
            
            <script type="text/javascript" src="index.js"></script>
        </head>
        
        <body>
        </body>
    </html>



Running tests in NodeJS
---------

If your code is written in a cross-platform manner, you can also run your tests in NodeJS. For that, first of all, install Siesta in the "node_modules"
folder of your project. Then, in the project, detect the platform and choose a correct project class, like this: 

    var Harness
    
    if (typeof process != 'undefined' && process.pid) {
        Harness = require('siesta');
    } else {
        Harness = new Siesta.Project.Browser()
    }

And finally, launch the project file in NodeJS:

    > node tests/index.js
    
You will see the same output as when running the test suite in Puppeteer. See also the "nodejs-browsers" example in the "/examples" folder.

In case of any failures in the test suite, the command will exit with non-zero exit code.


Buy this product
---------

Visit our store: <https://bryntum.com/store/siesta>


Support
---------

Ask question in our community forum: <https://www.bryntum.com/forum/viewforum.php?f=20>

Subscribers can post expedited questions in Premium Forum: <https://www.bryntum.com/forum/viewforum.php?f=21>

Please report any bugs through the web interface at <https://www.assembla.com/spaces/bryntum/support/tickets>


See also
---------

Siesta web-page: <https://bryntum.com/products/siesta>

Other Bryntum products: <https://bryntum.com/products>



Attribution
---------

This software contains icons from the following icon packs (licensed under Creative Common 2.5/3.0 Attribution licenses)

- <http://www.famfamfam.com/lab/icons/silk/>
- <http://led24.de/iconset/>
- <http://p.yusukekamiyamane.com/>
- <http://rrze-icon-set.berlios.de/index.html>
- <http://www.smashingmagazine.com/2009/05/20/flavour-extended-the-ultimate-icon-set-for-web-designers/>
- <http://www.doublejdesign.co.uk/products-page/icons/super-mono-icons/>
- <http://pixel-mixer.com/>

Thanks a lot to the authors of the respective icons packs.


COPYRIGHT AND LICENSE
---------

Copyright (c) 2009-2018, Bryntum & Nickolay Platonov

All rights reserved.