Intro
---------

Running the test suite manually in browsers is very convenient as it allows you to easily debug your tests. However, when
setting up a continuous integration process, it quickly becomes time-consuming - ideally you should to run the test suite
in each and every browser after (or before) each and every commit.

This guide describes how you can automate the launching of a Siesta test suite.

**Please note:** This functionality is only available in the Siesta Standard package.  


Running tests using Selenium WebDriver
---------

**Important**. Using the WebDriver requires some manual configuration steps for IE and Safari. Please refer to these 
pages <https://github.com/SeleniumHQ/selenium/wiki/InternetExplorerDriver> and <https://github.com/SeleniumHQ/selenium/wiki/SafariDriver>. 
Also, when running test suite in IE, make sure the IE window is focused (on top of other windows) and mouse cursor is outside of the IE window.

You can automate the launching of your test suite in several browsers, using Selenium WebDriver. 
WebDriver requires the presence of actual browsers in the operation system, installed in the default locations.

For browsers that supports headless mode (Chrome and Firefox) you can use `--headless` option to prevent opening of browser window (and interfering
with your normal work).   

On MacOS and Linux:

    > __SIESTA__/bin/webdriver http://yourproject/tests/index.html [OPTIONS]
    
On Windows:

    > __SIESTA__\bin\webdriver http://yourproject/tests/index.html [OPTIONS]
    
Here, the `__SIESTA__` placeholder is the path to your Siesta package. The launcher script `bin/webdriver` accepts the URL to 
your html wrapper for the Siesta project (`index.html`) and several optional options.

All options should start with double minus, e.g: `--browser chrome` or `--browser=chrome`. Please see the output of the `bin/webdriver --help` for 
all available options. Most important options are below:

- `help` - prints help message with all available options
- `browser browsername` - can be one of "firefox / chrome / ie / safari / edge"'.
- `headless` - launch the browser headless (not visible and not interfering with the current desktop).
- `max-workers` - maximum number of parallel testing "threads" that can be opened simultaneously. Please refer to a "Parallelization" section below.
- `include regexp`      - a regexp to filter the urls of tests. When provided, only the tests with urls matching this filter be executed
- `verbose` - will include the information about every individual assertions to the output. By default, only failed assertions will be shown
- `debug` - will enable various debugging messages
- `report-format` - the format of the report, see the "Reporting the results of a test suite execution in a structured format" section below
- `report-file` - the file to save the report to

You can also use the `--cap` switch to specify various Selenium capabilities for example "--cap browserName=firefox --cap platform=XP",
see <https://code.google.com/p/selenium/wiki/DesiredCapabilities>

In case of any failures in the test suite the command will exit with non-zero exit code. See "Exit codes" section for details.


Running tests using Puppeteer
---------

Puppeteer is an automation library for the headless Chrome browser. Puppeteer uses real rendering engine of the Chromium browser - 
you can trust the results from Puppeteer as if they were received from "real" Chrome.

Puppeteer will download a required Chrome version on the 1st launch. Further launches will be instant.

On MacOS and Linux:

    > __SIESTA__/bin/puppeteer http://yourproject/tests/index.html [OPTIONS]
    
On Windows:

    > __SIESTA__\bin\puppeteer http://yourproject/tests/index.html [OPTIONS]
    
Here, the `__SIESTA__` placeholder is the path to your Siesta package. The launcher script `bin/puppeteer` accepts the URL to your html wrapper for the Siesta project (`index.html`)
and several optional options.

Puppeteer supports almost the same set of options as the webdriver launcher, please refer to: `bin/puppeteer --help`


Running tests using remote Selenium WebDriver server
---------

You can launch the test suite (and receive the results) on one machine, but physically open the browsers on some other machine using RemoteWebDriver server.
To do that, first start the RemoteWebDriver server on target machine:

On MacOS and Linux:

    > __SIESTA__/bin/webdriver-server [OPTIONS]
    
On Windows:

    > __SIESTA__\bin\webdriver-server [OPTIONS]
 
"webdriver-server" is a very thin wrapper around "selenium-server-standalone-xx.jar" which just specifies the location of binaries for various browsers. It bypass
any command line options to that jar file. For example, to specify the port of server (default value is 4444), specify it as:

    > __SIESTA__/bin/webdriver-server -port 4444

For a list of available options for server, launch it with "-help" switch:

    > __SIESTA__/bin/webdriver-server -help

Also, please refer to: <https://github.com/SeleniumHQ/selenium/wiki/RemoteWebDriverServer> 

Then, when launching the test suite with "webdriver" launcher, specify an additional "--host" option:

    > __SIESTA__/bin/webdriver http://my.project.url/tests --host remote.webdriver.host --port 4444
    
You can use the "--cap" switch to specify various Selenium capabilities for example "--cap browserName=firefox --cap platform=XP",
see <https://code.google.com/p/selenium/wiki/DesiredCapabilities>


Configuration file
------------------

Configuration file can be specified with the "--config-file" command line option. If this option is not given, Siesta will 
automatically check for the presence of "siesta.json" file in the current directory or one of the parent directories. 
If such file presents it will be read.

Config file should contain JSON, with a bit less strict formatting rules. Notably, the // and /* */ comments are allowed, 
identifiers can be unquoted, strings can be quoted with single-quote character.
    
The format of config file:
    
    {
        // entries corresponds to project configs and overrides 
        // the values from the "project.configure()" call
        project  : {
            transparentEx       : true
        },
        
        // enrties corresponds to command line options, but *not* overrides them - 
        // options from the command line takes precendence
        cmd : {
            rerun-failed        : true
        }
    }
    
Command line options like `debug` or `verbose` should be specified with the boolean value `true/false`.
If command line option is repeatable, then you can specify an array with several values.

Native events
-------------

By default Siesta uses synthetic events simulation, which is very fast. However, in some cases, you may need to test some intrinsic browser
behavior which synthetic events can not reproduce. Most notable example will be testing the `:hover` CSS styles of some DOM elements.
In such case you can use {@link Siesta.Project.Browser#simulation native events simulation}

Parallelization
---------------

Siesta allows you to run the test suite in several parallel "threads" or "workers", speeding up the execution almost linearly by the
number of "workers". 

This can be done using the `--max-workers` option, which specifies the maximum number of "workers". 

There are some caveats though, related to focus handling. Even if several browser windows are opened simultaneously,
the focus can remain only in one of them, and all browsers "fights" for it (focus jumps randomly from one window to
another). This naturally leads to tests instability, since code often relies on some element being focused.
So, in general, the `--max-workers` can be used safely only for the non-visual tests. However see below.

- Cloud providers 

When using cloud-based infrastructure (SauceLabs or BrowserStack), each test page is running inside of the own VM, 
which guarantees the exclusive focus owning and makes it safe to use `--max-workers`. 

- XVFB

Linux only. When using Linux with the "Xvfb" package installed, one can add special command-line switch `--xvfb`, which launch
every browser window inside the separate virtual X desktop (with own focus handling). This makes it safe to specify
`--max-workers` more than 1 and speeds up the test suite execution by the number of workers! For this option to work,
the `Xvfb` is expected to be in PATH, same for the browser binaries, `google-chrome-stable` and `firefox`. This feature is 
now deprecated in favor of headless mode (which does essentially the same thing)

- Headless

The headless mode of the webdriver launcher provides an isolated environment for every browser window as well. It is usually safe
to use several workers in this case. Note, however, that currently headless Chrome is known to be very slow on Windows

**Important**. When value of `--max-workers` is more than 1, the order of tests execution is not defined. A test, that goes lower
in the `project.start()` list, can be executed before the test above it. This is simply because all tests are divided in several
"threads" and all threads are executed simultaneously. You should not rely on some test being run after another, instead, 
every test should execute standalone (allocate exclusive resources, perform all necessary setup).


Reports 
---------------------

You can export the results of a test suite execution in the structured format. To do that, provide the `--report-format` and `--report-file` 
options to the launcher. Supported formats are "HTML", "JSON", "JSONS" and "JUnit" (case insensitive). These options can be repeated 
several times, to generate several reports. 

The `JSON` and `JSONS` formats are almost identical, the `JSON` format contains the test results as a flat list, 
the `JSONS` contains results as a tree structure, based on the test groups information.

`HTML` report is basically a `JSONS` report, plus visualization files. It can be used to examine test suite execution results in the browser.
When using `HTML` report, the `--report-file` option actually specifies the *directory* for files (as this report consists from several files).
The `JSONS` report will be created as `report-data.json` file in that directory.

An example:

	siesta/bin/webdriver http://localhost/YourApplication/tests --report-format JSON --report-file foo.json

And this will generate a file named `foo.json` containing the following JSON structure:

    {
        "testSuiteName" : "Siesta self-hosting test suite",
        "startDate"     : 1343114314723,
        "endDate"       : 1343114315401,
        "passed"        : true,
        "testCases"     : [{
            "url"           : "010_sanity.t.js",
            "startDate"     : 1343114315390,
            "endDate"       : 1343114315396,
            "passed"        : true,
            "assertions"    : [{
                "passed"        : true,
                "description"   : "Siesta is here",
                "type"          : "Siesta.Result.Assertion"
            }, {
                "passed"        : true,
                "description"   : "Siesta.Test is here",
                "type"          : "Siesta.Result.Assertion"
            }, {
                "passed"        : true,
                "description"   : "Siesta.Project is here",
                "type"          : "Siesta.Result.Assertion"
            }, {
                "passed"        : false,
                "description"   : "Field 1 focused",
                "annotation"    : "Failed assertion `ok` at line 27 of keyevents/050_tab_key_focus2.t.js\nGot: false\nNeed \"truthy\" value",
                "sourceLine"    : "27",
                "name"          : "ok"
                "type"          : "Siesta.Result.Assertion"
            }]
        }]
    }

Using the JUnit report option looks like this:

	siesta/bin/webdriver http://localhost/YourApplication/tests --report-format junit --report-file foo.xml

This will generate a file named `foo.xml` containing the following JUnit XML structure:

	<testsuite errors="0" failures="1" hostname="localhost:8085" name="Ext Scheduler Test Suite" tests="2" time="3.594" timestamp="2012-06-06T08:55:21.520">
		
		<testcase classname="Bryntum.Test" name="lifecycle/040_schedulergrid.t.js" time="1.238">
			<failure message="Oops" type="FAIL"></failure>
		</testcase>

		<testcase classname="Bryntum.Test" name="lifecycle/042_schedulergrid_right_columns.t.js" time="0.818"></testcase>
	</testsuite>

 
Exit codes
----------

- 0 - All tests passed successfully
- 1 - Some tests failed
- 3 - No supported browsers available on this machine
- 4 - No tests to run (probably filter doesn't match any test url)
- 5 - Can't open project page
- 6 - Wrong command line arguments
- 8 - Exit after showing the Siesta version (when `--version` is provided on the command line)


Buy this product
---------

Visit our store: <https://bryntum.com/store/siesta>

Support
---------

Ask a question in our community forum: <https://www.bryntum.com/forum/viewforum.php?f=20>

Subscribers can post expedited questions in Premium Forum: <https://www.bryntum.com/forum/viewforum.php?f=21>

Please report any bugs through the web interface at <https://www.assembla.com/spaces/bryntum/support/tickets>


See also
---------

Siesta web-page: <https://bryntum.com/products/siesta>

Other Bryntum products: <https://bryntum.com/products>


COPYRIGHT AND LICENSE
---------

Copyright (c) 2009-2018, Bryntum AB & Nickolay Platonov

All rights reserved.